#include <assert.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <fcntl.h>
#include <linux/fb.h>
#include <sys/mman.h>

#include "startup_image.h"

int main()
{
        int fbfd, st, x, y, xoffset, yoffset;
        struct fb_var_screeninfo vinfo;
        struct fb_fix_screeninfo finfo;
	long int screensize;
	char *fbp;

        /* Open the file for reading and writing */
        fbfd = open("/dev/fb0", O_RDWR);
	assert(fbfd);
        /* Get fixed screen information */
        st = ioctl(fbfd, FBIOGET_FSCREENINFO, &finfo);
	assert(!st);
	assert(finfo.type == FB_TYPE_PACKED_PIXELS);
	assert(finfo.visual == FB_VISUAL_TRUECOLOR);
        /* Get variable screen information */
        st = ioctl(fbfd, FBIOGET_VSCREENINFO, &vinfo);
	assert(!st);
	assert(vinfo.bits_per_pixel == 16);
	assert(!vinfo.grayscale);
	/* 565 color */
	assert(vinfo.red.offset == 11);
	assert(vinfo.red.length == 5);
	assert(!vinfo.red.msb_right);
	assert(vinfo.green.offset == 5);
	assert(vinfo.green.length == 6);
	assert(!vinfo.green.msb_right);
	assert(vinfo.blue.offset == 0);
	assert(vinfo.blue.length == 5);
	assert(!vinfo.blue.msb_right);
	assert(vinfo.transp.length == 0); /* no alpha */
        /* Figure out the size of the screen in bytes */
        screensize = vinfo.xres * vinfo.yres * vinfo.bits_per_pixel / 8;
        /* Map the device to memory */
        fbp = (char *)mmap(0, screensize, PROT_READ | PROT_WRITE, MAP_SHARED,
                fbfd, 0);
	assert(fbp != MAP_FAILED);

	/* center the pic */
	xoffset = ((int)vinfo.xres - (int)startup_image.width) / 2;
	yoffset = ((int)vinfo.yres - (int)startup_image.height) / 2;

	/* copy the pic */
	for (y=0; y < startup_image.height; y++) {
	    int len, x0, x1, sx0, sx1, sy;
	    sy = y + yoffset;
	    if (sy < 0 || sy >= vinfo.yres) continue;
	    /* memcpy a full row */
	    x0 = 0; sx0 = x0 + xoffset;
	    x1 = startup_image.width; sx1 = x1 + xoffset;
	    if (sx0 < 0) { x0 -= sx0; sx0 = 0; }
	    if (sx1 > vinfo.xres) { int diff = vinfo.xres - sx1; x1 -= diff; sx1 -= diff; }
	    memcpy(fbp + (sx0 + vinfo.xoffset) * (vinfo.bits_per_pixel/8) +
		   (sy + vinfo.yoffset) * finfo.line_length,
		   startup_image.pixel_data + x0 + y * startup_image.width,
		   2 * (x1 - x0));
	}
	munmap(fbp, screensize);
	close(fbfd);
}
