# stub antitheft daemon
import os, sys
from clone import * # pyrex binding of clone syscall

# this is the interface exported by the antitheft client (ATC)

def run(activated, serial_number, uuid, schoolserver_fqdn,
        write_lease_cb, run_init_callback):
    # do initial activation, if we're currently unactivated
    from activate import activate
    if not activated:
        key = activate(serial_number, uuid)
        if key is not None:
            write_lease_cb(key) # write new lease to disk
            # XXX pause a second to let user see the status indication?
        else:
            # activation failed.  shutdown now.
            from initutil import activation_fail
            activation_fail(serial_number)
            sys.exit(1) # we shouldn't reach this point
            return # we shouldn't reach this point

    # okay, we're activated.  fork and invoke the run_init_callback in pid 2
    # note that we need to clone() with CLONE_FS so that when pid 2 chroots
    # we follow along (since the kernel threads also share our fs context,
    # and at the moment we want modprobe, etc, to happen in the 'real' fs)
    if clone(CLONE_FS) == 0:
        # hey, we're the child!
        # let's invoke the callback.
        # too bad we're not properly tail recursive. =(
        return run_init_callback()

    # we're the parent.
    os.close(0) # close stdin/out/err
    os.close(1)
    os.close(2)

    # set our process title, so that people know that PID 1 is the ATC
    from proctitle import setproctitle
    setproctitle("oatc") # olpc antitheft client

    # we need to reap zombies
    import signal
    def zombie_handler(signum, frame):
        try:
            while True:
                pid, exitcode = os.waitpid(-1, os.WNOHANG)
                if pid == 0: break # no more children to wait for
        except OSError:
            pass # no more children to wait for
    signal.signal(signal.SIGCHLD, zombie_handler)

    # let's do ATC stuff...
    import time
    while True:
        time.sleep(3600) # ...or not.

    # if we ever return, PID 1 will end and the kernel will panic.
    sys.exit(77)
