#!/usr/bin/python2.5
## Add an '-i' to the #! line to enable the interactive shell.
## Remove the -i for deployment, to ensure that an
## error in the script doesn't land us in an interactive shell.
from __future__ import division, with_statement
from initutil import *
from activate import activate
import antitheft
import os, os.path, sys, time

debug = False # set to false for deployment

# let's notify console that we've reached this point and python has launched
print "Hello, (children of the) world!"

# do initial system setup
mount_proc_sys()

# parse kernel command line
xo = True
boot_backup = False
interactive = False
do_activate = False
frs_upgrade = False
## In addition to cmdline parsing, we also look at /ofw/chosen/bootpath
## which is something like:
##   /pci/nandflash@c:\boot\vmlinuz//jffs2-file-system:\boot\vmlinuz
## Details at http://wiki.laptop.org/go/Firmware_Security
##
for arg in open("/proc/cmdline").read().split():
    if False: pass
    elif arg=='activate': do_activate = True # booting an unactivated machine
    # upgrade support.
    elif arg=='altboot': boot_backup = True
    # for debugging, accept 'int'/'noint' option to drop into a python shell.
    elif arg=='int': interactive = True
    # when booting into the emulator, 'emu' should be on the command line.
    # this way we don't try to mount a non-existent flash device.
    elif arg=='emu': xo = False # set this if booting in an emulator
    # XXX: would be nice to look for a 'root=' option here, for pgf

# be nice to emulators by checking to see if we're on a real XO.
# if we are, get the serial number and UUID
arch, serial_number, uuid, bootpath, ak = "qemu", None, None, None, False
if xo:
    try:
        safe_sh("mount -t promfs promfs /ofw")
        arch = trim_ofw(read_ofw('architecture'))
        serial_number = trim_ofw(read_ofw('mfg-data/SN'))
        uuid = trim_ofw(read_ofw('mfg-data/U#'))
        bootpath = trim_ofw(read_ofw('chosen/bootpath'))
        ak = exists_ofw('mfg-data/ak')
        safe_sh("umount /ofw")
    except:
        xo = False # something went wrong, not a real XO.
# preproduction machines don't have serial_number or UUID, but arch is correct
# (B1s have U# but not SN, machines w/o mfg-data don't have U# or SN.)
# emuators have none of these.
if (serial_number is None or uuid is None) and arch != 'OLPC':
    # we're not really on an XO.  Probably an emulator.
    xo = False
serial_number = serial_number or 'SHF00000000'
uuid = uuid or '00000000-0000-0000-0000-000000000000'
# add a bit of randomness to the pool (trac #7134)
open('/dev/urandom','w').write('%s/%s' % (serial_number, uuid))
# use the hardware RNG to generate some more (trac #7213)
try:
    open('/dev/urandom','w').write(open('/dev/hwrng').read(1024))
except:
    pass # don't fail if we're running in an emulator.
# are we booting from an alternate image?
if bootpath and '\\boot-alt\\' in bootpath:
    boot_backup = True

### check for activation code, perform activation if necessary ####

if bootpath and ('\\actos.zip' in bootpath or '\\actrd.zip' in bootpath):
    # in theory, bootpath should have the name of the *kernel* booted
    # which would be actos.  But some firmwares inadvertently pass the
    # ramdisk name instead (actrd).
    do_activate = True
if (not xo) or ak:
    do_activate = False

ACTIVATE_KEY = '/sysroot/security/lease.sig'
def lease_writer(lease):
    """Write new key to disk after initial activation."""
    with root_mounted(xo, bootpath):
        with writable_root():
            if not os.path.exists('/sysroot/security'):
                os.mkdir('/sysroot/security')
            with open(ACTIVATE_KEY, 'w') as f:
                f.write(lease)

def run_init():
    """Invoke the next stage of early boot init, as PID 2."""
    ########### MACHINE IS ACTIVATED #########
    # it's safe to mount root now.

    ## clean up variables
    serial_number, uuid, key, arch = None, None, None, None

    ## load modules and mount root on /sysroot.
    root_mounted(xo, bootpath).__enter__() # hack to re-mount root

    # check private security dir
    check_stolen()
        
    # launch pretty boot just as soon as possible
    start_boot_animation('/sysroot')

    # if we're booting from the backup image, frob the symlink
    current = frob_symlink(boot_backup)
    if current:
        newroot = '/sysroot/versions/run/'+current
        # launch pretty boot just as soon as possible (upgradable case)
        start_boot_animation(newroot)

    if not interactive:
        if current:
            boot_run_xo(current)
        else:
            boot_xo()

if xo and frs_upgrade and boot_backup and \
       bootpath.startswith('/pci/nandflash@'):
    try_frs_upgrade(xo, bootpath) # try special MP-to-FRS upgrade

# invoke the antitheft client
antitheft.run(not do_activate, serial_number, uuid,
              'schoolserver.laptop.org', # FIXME needs real fqdn
              lease_writer, run_init)

# we must be interactive.  let's be friendly.
quit = "Use boot_xo() to continue boot."
exit = quit
print "Interactive python shell."
print quit
# unfreeze dcon if it is frozen.
try:
    with open('/sys/devices/platform/dcon/freeze','w') as f:
        f.write("0")
except: pass

# dropping off end will leave us in interactive shell, due to -i arg in #! line
