#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2007-2008 One Laptop Per Child Association, Inc.
# Written by Ivan Krstić

# This program is free software; you can redistribute it and/or modify
# it under the terms of version 2 (and no other version) of the GNU
# General Public License as published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

import sys
import time

from bitfrost.leases.core import find_lease
from bitfrost.leases.crypto import verify_lease
from bitfrost.leases.errors import *

# Default channels for obtaining a lease
channels = ['file:///keylist', 'https://localhost/leases']

# Number of seconds to sleep between failed attempts to acquire a lease.
LEASE_SLEEP_DELAY = 5

# Maximum lease file size. Limit to prevent DoS.
LEASE_MAX_SIZE = 10 * 1024 * 1024

# Take one optional command-line parameter: comma-delimited channel list,
# no spaces.
if len(sys.argv) > 1:
    channels = sys.argv[1].lower().split(',')

# Obtain this machine's serial number and UUID. Strip last two
# characters if they're a newline and a null-byte. See OLPC trac #2085.
try:
    this_sn = file('/ofw/serial-number').read(32).rstrip("\n\0")
    this_uuid = file('/ofw/mfg-data/U#').read(64).rstrip("\n\0")
except:
    print "Can't determine SN or UUID. Something's terribly wrong, panic."
    sys.exit(1)

lease_verified = False
while not lease_verified:
    for channel in channels:
        if channel.startswith('file://'):
            path = channel[len('file://'):]
            print "Trying file channel: [%s]." % path
            try:
                if find_lease(this_sn, this_uuid, \
                              file(path).read(LEASE_MAX_SIZE)):
                    lease_verified = True
            except:
                raise
                print "Error: %s, skipping to next channel." % sys.exc_info()[0]
            
        elif channel.startswith('https://'):
            print Exception("HTTPS channels not yet implemented.")

        else:
            print "Skipping unsupported channel [%s]." % channel

    if lease_verified:
        break
    
    print "Failed to obtain a valid lease through any channel. Sleeping %d." \
           % LEASE_SLEEP_DELAY
    try:
        time.sleep(LEASE_SLEEP_DELAY)
    except:
        print "Woken up by hand, will try again."

print "Successfully verified lease for [%s]." % this_sn
